if (!('contains' in String.prototype)) {
    String.prototype.contains = function(str, startIndex) {
        return ''.indexOf.call(this, str, startIndex) !== -1;
    };
}

if (typeof String.prototype.startsWith != 'function') {
    String.prototype.startsWith = function(prefix) {
        return this.slice(0, prefix.length) == prefix;
    };
}

if (typeof String.prototype.endsWith != 'function') {
    String.prototype.endsWith = function(suffix) {
        return this.slice(-suffix.length) == suffix;
    };
}

var ngp = require('../ecxngp');
var vadpProvider = require('../vadp-provider');
var vadpUpdateProvider = require('../vadp-update-provider');

var vadpRouter = ngp.express.Router();

const postVadplicitQueries = {
    action: ['installandregister','register','edit']
};

//middleware which is specific to the vadp router
vadpRouter.use(function timeLog(req, res, next) {
    ngp.logMsg('info','Handling /ecxngp/vadp request Time(%s) METHOD(%s)', Date.now().toString(), req.method.toString());
    next();
});

/**
 * @swagger
 * /ngp/vadp:
 *   post:
 *     tags: ["VADP"]
 *     summary: Add a new VADP proxy
 *     description: Adds a new VAPD proxy by either registering an existing VADP proxy or installing and registering a new VADP proxy
 *     requestBody:
 *       description: A JSON representation of the VADP proxy to add
 *       required: true
 *     consumes: 
 *       - application/json
 *     parameters:
 *       - in: query
 *         name: action
 *         required: true
 *         type: string
 *         enum: [register, installandregister]
 *         description: >
 *           The action to execute:
 *             * `register` - Register an existing VADP proxy
 *             * `installandregister` - Install and register a new VADP proxy
 *       - in: body
 *         name: vadp
 *         description: >
 *           The VADP proxy to add.<br /><br />Note that the `identityId` field can be replaced with a `string` value of the 
 *           identity ID of the user associated with the VADP proxy if the identity ID exists (rather than 
 *           supplying the `username` and `password` fields). For example, `"identityId": "1001"`.
 *         required: true
 *         schema:
 *           type: object
 *           properties:
 *             pushinstall:
 *               type: object
 *               properties:
 *                 hostAddress:
 *                   type: string
 *             identityId:
 *               type: object
 *               properties:
 *                 username:
 *                   type: string
 *                 password:
 *                   type: string
 *             registration:
 *               type: object
 *               properties:
 *                 siteId:
 *                   type: string
 *     responses:
 *       201:
 *         description: The supplied VADP was successfully added
 *       500:
 *         description: Addition of the supplied VADP proxy failed (see response body for more information)
 *       400:
 *         description: The supplied VADP does not include a `hostAddress` entry
 */
vadpRouter.post('/', function(req, res) {
    var vadpQueryValidationResult = ngp.validateQs(req, postVadplicitQueries);

    if ( vadpQueryValidationResult.rc == 2 )
    {
        ngp.logMsg('warn', 'Detected invalid value among keys=' + JSON.stringify(vadpQueryValidationResult.keyVals));
        ngp.returnBadRequestError(res);
        return;
    }

    if ( vadpQueryValidationResult.rc == 0 ) {
        ngp.logMsg('warn', 'Detected no query parameters');
        ngp.returnBadRequestError(res);
        return;
    }
    var stepsStream = null;

    var skipInstall = 'false';

    if (req.query['action']){
        if (req.query['action'] == 'installandregister') {
                     
            stepsStream = vadpProvider.init(req, res, 'installAndRegister', skipInstall);
            ngp.startFunc(req, res, stepsStream);       
        }
        else if (req.query['action'] == 'register') {
           
            skipInstall = 'true';
            
            stepsStream = vadpProvider.init(req, res, 'installAndRegister', skipInstall);
            ngp.startFunc(req, res, stepsStream);       
        }
        else {
            ngp.returnBadRequestError(res);
            return;            
        }
    }
});

vadpRouter.put('/:id', function(req, res) {
    var vadpQueryValidationResult = ngp.validateQs(req, postVadplicitQueries);

    if ( vadpQueryValidationResult.rc == 2 )
    {
        ngp.logMsg('warn', 'Detected invalid value among keys=' + JSON.stringify(vadpQueryValidationResult.keyVals));
        ngp.returnBadRequestError(res);
        return;
    }

    if ( vadpQueryValidationResult.rc == 0 ) {
        ngp.logMsg('warn', 'Detected no query parameters');
        ngp.returnBadRequestError(res);
        return;
    }
    var stepsStream = null;

    if (req.query['action']){
        if (req.query['action'] == 'edit') {           
            stepsStream = vadpUpdateProvider.init(req, res);
            ngp.startFunc(req, res, stepsStream);       
        }
        else {
            ngp.returnBadRequestError(res);
            return;            
        }
    }
});

module.exports = vadpRouter;